package xpref.datatypes;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.lang.instrument.IllegalClassFormatException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.LinkedList;

import javax.swing.ButtonGroup;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import xpref.XPref;
import xpref.XPrefGUI;
import xpref.util.XMLUtils;

public class CollectionDatatype {
	/**
	 * The default value for this data type.
	 */
	protected static String defaultValue = "";

	/**
	 * Static method to serialize the given element.
	 * 
	 * @param element
	 *            the element to serialize
	 * 
	 * @return the serialized element as string
	 */
	public static String serializeElement(Object element) {
		if(element == null) {
			element = CollectionDatatype.defaultValue;
		}

		return (String) element;
	}

	/**
	 * Static method to parse the string representation of the element to the
	 * right data type.
	 * 
	 * @param optionNode
	 *            the option node to parse the element for
	 * @param element
	 *            the string representation of the element
	 * 
	 * @return the element as instance of the right data type
	 */
	public static String parseElement(Node optionNode, String element) throws Exception {
		if(element == null) {
			return CollectionDatatype.defaultValue;
		}
		else {
			Collection<Object> coBoElements = CollectionDatatype.getCollection(optionNode);

			if(coBoElements.contains(element)) {
				return element;
			}
			else {
				throw new NoSuchFieldException("The Collection for the id '" + XMLUtils.getAttributeValue(optionNode, "id") + "' has no member '" + element + "'!");
			}
		}
	}

	/**
	 * Static method to get the visual representation of the element for
	 * XPrefGUI.
	 * 
	 * @param element
	 *            the element to represent
	 * @param optionNode
	 *            node of the element
	 * @param xPrefGUI
	 *            instance of the XPrefGUI
	 * @param pageName
	 *            name of the page this value is on
	 * 
	 * @return the panel holding the visual representation
	 * 
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws NoSuchMethodException
	 *             if the second part of the source is invalid
	 * @throws ClassNotFoundException
	 *             if the first part of the source is invalid
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 */
	public static JPanel getVisualRepresentation(final String nodeID, Object element, Node optionNode, final XPrefGUI xPrefGUI, final String pageName, final Integer index) throws SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchMethodException, IllegalAccessException, InvocationTargetException {
		// wrap check box in a panel...
		JPanel panel = new JPanel();

		Collection<Object> coBoElements = CollectionDatatype.getCollection(optionNode);


		String elem = null;

		if(element == null) {
			elem = CollectionDatatype.defaultValue;
		}
		else {
			elem = (String) element;
		}


		int selectIndex = CollectionDatatype.indexOf(coBoElements, elem);

		if(selectIndex == -1) {
			selectIndex = CollectionDatatype.indexOf(coBoElements, XMLUtils.getAttributeValue(optionNode, "value"));
		}

		if(selectIndex == -1) {
			selectIndex = 0;
		}


		String type = XMLUtils.getAttributeValue(optionNode, "parameters");

		// radio buttons...
		if(type != null && type.startsWith("radiobuttons")) {
			if(type.endsWith("=vertical")) {
				panel.setLayout(new GridLayout(0, 1));
			}
			else {
				panel.setLayout(new GridLayout(1, 0));
			}

			ButtonGroup group = new ButtonGroup();
			int i = 0;

			for(final Object collectionElem : coBoElements) {
				JRadioButton radioButton = new JRadioButton(collectionElem.toString());
				radioButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						xPrefGUI.valueChanged(pageName, nodeID, collectionElem.toString(), index);
					}
				});

				if(i == selectIndex) {
					radioButton.setSelected(true);
				}

				group.add(radioButton);
				panel.add(radioButton);
				i += 1;
			}
		}
		else { // combo box...
			final JComboBox optionElement = new JComboBox(coBoElements.toArray());
			optionElement.setSelectedIndex(selectIndex);
			optionElement.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent arg0) {
					String saveValue = (String) optionElement.getSelectedItem(); // get new state

					xPrefGUI.valueChanged(pageName, nodeID, saveValue, index);
				}
			});

			panel.add(optionElement);
		}

		return panel;
	}

	/**
	 * Static method to get the first value of the given id as instance of the
	 * proper data type.
	 * 
	 * @param id
	 *            the id to get the value for
	 * 
	 * @return the element as instance of the right data type
	 * 
	 * @throws IOException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws XPathExpressionException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws IllegalClassFormatException
	 * @throws NoSuchMethodException
	 * @throws NoSuchFieldException
	 * @throws ClassNotFoundException
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 */
	public static String getFirstValue(String id) throws XPathExpressionException, SAXException, ParserConfigurationException, IOException, SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchFieldException, NoSuchMethodException, IllegalClassFormatException, IllegalAccessException, InvocationTargetException {
		LinkedList<String> list = CollectionDatatype.getValues(id);

		return list.get(0);
	}

	/**
	 * Static method to get the values to the given id as instances of the
	 * proper data type.
	 * 
	 * @param id
	 *            the id to get the values for
	 * 
	 * @return the elements as list of instances of the right data type
	 * 
	 * @throws IOException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws XPathExpressionException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws IllegalClassFormatException
	 * @throws NoSuchMethodException
	 * @throws NoSuchFieldException
	 * @throws ClassNotFoundException
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 */
	public static LinkedList<String> getValues(String id) throws XPathExpressionException, SAXException, ParserConfigurationException, IOException, SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchFieldException, NoSuchMethodException, IllegalClassFormatException, IllegalAccessException, InvocationTargetException {
		LinkedList<String> collectionList = new LinkedList<String>();

		for(Object obj : XPref.getInstance().getValues(id)) {
			collectionList.add((String) obj);
		}

		return collectionList;
	}

	/**
	 * Internal method to get the index of an object in a collection
	 * 
	 * @param collection
	 *            collection to find the object in
	 * @param element
	 *            element to find in the collection
	 * 
	 * @return the index of the element in the collection; -1 if the element is
	 *         not in the collection
	 */
	private static int indexOf(Collection<Object> collection, Object element) {
		int index = -1;

		for(Object obj : collection) {
			index++;

			if(obj.equals(element)) {
				break;
			}
		}

		return index;
	}

	/**
	 * Internal method to get the collection associated with the option node.
	 * 
	 * @param optionNode
	 *            the option node to get the collection for
	 * 
	 * @return the collection of the option node
	 * 
	 * @throws ClassNotFoundException
	 *             if the first part of the source is invalid
	 * @throws NoSuchMethodException
	 *             if the second part of the source is invalid
	 * @throws SecurityException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws IllegalArgumentException
	 */
	@SuppressWarnings("unchecked")
	private static Collection<Object> getCollection(Node optionNode) throws ClassNotFoundException, SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		// get source...
		String source = XMLUtils.getAttributeValue(optionNode, "source");

		// create empty list for JComboBox...
		Collection<Object> coBoElements = null;

		if(source != null) { // if javaList entry exists...
			String[] datatypePart = source.split(" "); // split class name and method name

			Class<?> c = Class.forName(datatypePart[0]); // get class

			Method m = c.getMethod(datatypePart[1]); // get method

			coBoElements = (Collection<Object>) m.invoke(c); // invoke method
		}

		return coBoElements;
	}

	/**
	 * Static method to describe the data type.
	 * 
	 * @return Description of the data type.
	 */
	public static String description() {
		return "Element out of a list of Collection values";
	}

	/**
	 * Static method to describe the usage for this option.
	 * 
	 * @param optionNode
	 *            the option node to show the description for
	 * 
	 * @return the description string
	 * 
	 * @throws ClassNotFoundException
	 *             if the first part of the source is invalid
	 * @throws NoSuchMethodException
	 *             if the second part of the source is invalid
	 * @throws SecurityException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws IllegalArgumentException
	 */
	@SuppressWarnings("unchecked")
	public static String usage(Node optionNode) throws ClassNotFoundException, SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException {
		StringBuffer specific = new StringBuffer();

		// get source...
		String source = XMLUtils.getAttributeValue(optionNode, "source");

		if(source != null) { // if javaList entry exists...
			String[] datatypePart = source.split(" "); // split class name and method name

			Class<?> c = Class.forName(datatypePart[0]); // get class

			Method m = c.getMethod(datatypePart[1]); // get method

			Object[] coBoElements = ((Collection<Object>) m.invoke(c)).toArray(); // invoke method

			for(int i = 0; i < coBoElements.length; i += 1) {
				specific.append(coBoElements[i].toString());

				if(i < coBoElements.length-1) {
					specific.append(" -- ");
				}
			}
		}

		return specific.toString();
	}

	/**
	 * Static method to get the name for this data type. The name can depend on
	 * some properties of the node it is used for.
	 * 
	 * @param optionNode
	 *            the node this data type is used for
	 * 
	 * @return the name of the data type
	 */
	public static String getName(Node optionNode) {
		// get source...
		String source = XMLUtils.getAttributeValue(optionNode, "source");

		return EnumDatatype.class.getName() + " (" + source + ")";
	}
}