package xpref.datatypes;

import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.IOException;
import java.lang.instrument.IllegalClassFormatException;
import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import xpref.XPref;
import xpref.XPrefGUI;

public class StringDatatype {
	/**
	 * The default value for this data type.
	 */
	protected static String defaultValue = "";

	/**
	 * Static method to serialize the given element.
	 * 
	 * @param element
	 *            the element to serialize
	 * 
	 * @return the serialized element as string
	 */
	public static String serializeElement(Object element) {
		if(element == null) {
			element = StringDatatype.defaultValue;
		}

		return (String) element;
	}

	/**
	 * Static method to parse the string representation of the element to the
	 * right data type.
	 * 
	 * @param optionNode
	 *            the option node to parse the element for
	 * @param element
	 *            the string representation of the element
	 * 
	 * @return the element as instance of the right data type
	 */
	public static String parseElement(Node optionNode, String element) {
		if(element == null) {
			return StringDatatype.defaultValue;
		}
		else {
			return element;
		}
	}

	/**
	 * Static method to get the visual representation of the element for
	 * XPrefGUI.
	 * 
	 * @param element
	 *            the element to represent
	 * @param optionNode
	 *            node of the element
	 * @param xPrefGUI
	 *            instance of the XPrefGUI
	 * @param pageName
	 *            name of the page this value is on
	 * 
	 * @return the panel holding the visual representation
	 */
	public static JPanel getVisualRepresentation(final String nodeID, Object element, Node optionNode, final XPrefGUI xPrefGUI, final String pageName, final Integer index) {
		String elem = "";

		if(element == null) {
			elem = StringDatatype.defaultValue;
		}
		else {
			elem = (String) element;
		}

		// create check box for the boolean...
		final JTextField optionElement = new JTextField(elem, 30);
		optionElement.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {}

			public void focusLost(FocusEvent e) {
				String content = optionElement.getText();

				xPrefGUI.valueChanged(pageName, nodeID, StringDatatype.serializeElement(content), index);
			}
		});

		// wrap check box in a panel...
		JPanel panel = new JPanel();
		panel.add(optionElement);

		return panel;
	}

	/**
	 * Static method to get the first value of the given id as instance of the
	 * proper data type.
	 * 
	 * @param id
	 *            the id to get the value for
	 * 
	 * @return the element as instance of the right data type
	 * 
	 * @throws IOException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws XPathExpressionException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws IllegalClassFormatException
	 * @throws NoSuchMethodException
	 * @throws NoSuchFieldException
	 * @throws ClassNotFoundException
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 */
	public static String getFirstValue(String id) throws XPathExpressionException, SAXException, ParserConfigurationException, IOException, SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchFieldException, NoSuchMethodException, IllegalClassFormatException, IllegalAccessException, InvocationTargetException {
		LinkedList<String> list = StringDatatype.getValues(id);

		return list.get(0);
	}

	/**
	 * Static method to get the values to the given id as instances of the
	 * proper data type.
	 * 
	 * @param id
	 *            the id to get the values for
	 * 
	 * @return the elements as list of instances of the right data type
	 * 
	 * @throws IOException
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws XPathExpressionException
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws IllegalClassFormatException
	 * @throws NoSuchMethodException
	 * @throws NoSuchFieldException
	 * @throws ClassNotFoundException
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 */
	public static LinkedList<String> getValues(String id) throws XPathExpressionException, SAXException, ParserConfigurationException, IOException, SecurityException, IllegalArgumentException, ClassNotFoundException, NoSuchFieldException, NoSuchMethodException, IllegalClassFormatException, IllegalAccessException, InvocationTargetException {
		LinkedList<String> list = new LinkedList<String>();

		for(Object obj : XPref.getInstance().getValues(id)) {
			list.add((String) obj);
		}

		return list;
	}

	/**
	 * Static method to describe the data type.
	 * 
	 * @return Description of the data type.
	 */
	public static String description() {
		return "String value";
	}

	/**
	 * Static method to describe the usage for this option.
	 * 
	 * @param optionNode
	 *            the option node to show the description for
	 * 
	 * @return the description string
	 */
	public static String usage(Node optionNode) {
		return "any string value";
	}

	/**
	 * Static method to get the name for this data type. The name can depend on
	 * some properties of the node it is used for.
	 * 
	 * @param optionNode
	 *            the node this data type is used for
	 * 
	 * @return the name of the data type
	 */
	public static String getName(Node optionNode) {
		return StringDatatype.class.getName();
	}
}