package xpref.formatter;

import java.lang.reflect.InvocationTargetException;
import java.util.LinkedList;

import javax.xml.xpath.XPathExpressionException;

import xpref.util.Tupel;

public class PlainTextFormatter implements IFormatter {
	private StringBuffer content = new StringBuffer();

	/**
	 * Method to reset the formatter to make it reusable.
	 */
	public void reset() {
		this.content = new StringBuffer();
	}

	/**
	 * Method to add text at the top of the output.
	 * 
	 * @param text
	 *            the text to add
	 */
	public void addText(String text) {
		this.content.append(text);
	}

	/**
	 * Method to add a new block for an option.
	 * 
	 * @param ID
	 *            the option id
	 * @param name
	 *            the option name
	 * @param dataType
	 *            the data type of the option
	 * @param defaultValue
	 *            the default value of the option
	 * @param helpText
	 *            the help text of the option
	 * @param possibleValues
	 *            the possible values of the option
	 * @param description
	 *            the description of the option
	 * 
	 * @throws XPathExpressionException
	 *             if there is an error in the XPath expressions
	 * @throws NoSuchMethodException
	 *             if the data type is invalid
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 * @throws ClassNotFoundException
	 * @throws SecurityException
	 * @throws IllegalArgumentException
	 */
	public void newBlock(String ID, String name, String dataType, String defaultValue, String helpText, String possibleValues, String description) throws XPathExpressionException, IllegalArgumentException, SecurityException, ClassNotFoundException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		this.newBlock(ID, name, dataType, defaultValue, helpText, possibleValues, description, new LinkedList<Tupel<Class<?>, String>>());
	}

	/**
	 * Method to add a new block for an option.
	 * 
	 * @param ID
	 *            the option id
	 * @param name
	 *            the option name
	 * @param dataType
	 *            the data type of the option
	 * @param defaultValue
	 *            the default value of the option
	 * @param helpText
	 *            the help text of the option
	 * @param possibleValues
	 *            the possible values of the option
	 * @param description
	 *            the description of the option
	 * @param possibleSequenceValues
	 *            possible values for the option (only defined if the option is
	 *            a sequence
	 * 
	 * @throws XPathExpressionException
	 *             if there is an error in the XPath expressions
	 * @throws NoSuchMethodException
	 *             if the data type is invalid
	 * @throws IllegalArgumentException
	 * @throws SecurityException
	 * @throws ClassNotFoundException
	 * @throws IllegalAccessException
	 * @throws InvocationTargetException
	 */
	public void newBlock(String ID, String name, String dataType, String defaultValue, String helpText, String possibleValues, String description, LinkedList<Tupel<Class<?>, String>> possibleSequenceValues) throws XPathExpressionException, IllegalArgumentException, SecurityException, ClassNotFoundException, IllegalAccessException, InvocationTargetException, NoSuchMethodException {
		if(!description.equals("")) {
			description += "; ";
		}

		this.content.append("ID: " + ID + " --- Name: " + name + " --- Datatype: " + dataType + "\n" +
				"Default value: " + defaultValue + "\n" +
				"Possible values: " + description + possibleValues + "\n");

		for(Tupel<Class<?>, String> t : possibleSequenceValues) {
			description = (String) t.first.getMethod("description").invoke(t.first);

			this.content.append("  " + t.first.getName() + ": " + description + "; " + t.second + "\n");
		}

		this.content.append("Helptext: " + helpText + "\n\n");
	}

	/**
	 * Method to get the formatted content.
	 * 
	 * @return the formatted content
	 */
	public String getContent() {
		return this.content.toString();
	}
}