package xpref.util;

import java.util.Iterator;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class SimpleXPath {
	/**
	 * Singleton object for SimpleXPath.
	 */
	private static SimpleXPath instance = null;

	/**
	 * Internal XPath object.
	 */
	private XPath xPath = null;

	/**
	 * Public method to get the singleton instance of SimpleXPath.
	 * 
	 * @return SimpleXPath instance
	 */
	public static SimpleXPath getInstance() {
		if(SimpleXPath.instance == null) {
			SimpleXPath.instance = new SimpleXPath();
		}

		return SimpleXPath.instance;
	}

	/**
	 * Private Constructor to initialize the XPath object.
	 */
	private SimpleXPath() {
		this.xPath = XPathFactory.newInstance().newXPath();
		this.xPath.setNamespaceContext(new NamespaceContext() {
			public String getNamespaceURI(String prefix) {
				if(prefix == null) {
					throw new NullPointerException("Null prefix");
				}
				else if("svg".equals(prefix)) {
					return "http://www.w3.org/2000/svg";
				}
				else if("xml".equals(prefix)) {
					return XMLConstants.XML_NS_URI;
				}

				return XMLConstants.NULL_NS_URI;
			}

			public String getPrefix(String uri) {
				throw new UnsupportedOperationException();
			}

			public Iterator<?> getPrefixes(String uri) {
				throw new UnsupportedOperationException();
			}
		});
	}

	public String getString(String expression, Object source) throws XPathExpressionException {
		return (String) this.xPath.evaluate(expression, source, XPathConstants.STRING);
	}

	public Node getNode(String expression, Object source) throws XPathExpressionException {
		return (Node) this.xPath.evaluate(expression, source, XPathConstants.NODE);
	}

	public NodeList getNodeSet(String expression, Object source) throws XPathExpressionException {
		return (NodeList) this.xPath.evaluate(expression, source, XPathConstants.NODESET);
	}
}