package xpref.util;

import java.awt.Dimension;
import java.util.Hashtable;

import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

public class Tree extends JTree {
	/**
	 * Serial version UID.
	 */
	private static final long serialVersionUID = -917413451692671328L;

	/**
	 * Holds the width of the tree.
	 */
	private int treeWidth = 160;

	/**
	 * Holds the root node of the Tree.
	 */
	private TreeNode rootNode;

	/**
	 * Maps the JTree-Path objects to the pageIDs.
	 */
	private Hashtable<String, TreePath> treePathMapping = new Hashtable<String, TreePath>();

	/**
	 * Holds the current selected tree node
	 */
	private TreeNode currentNode = null;

	/**
	 * Initiates the JTree.
	 * 
	 * @param rootNode
	 *            root node of the tree
	 */
	public Tree(TreeNode rootNode) {
		super(rootNode);

		this.rootNode = rootNode;

		this.currentNode = rootNode;
		this.setRootVisible(false);
		this.setPreferredSize(new Dimension(this.treeWidth, this.getPreferredSize().height));
	}

	public TreeNode setCurrentNode(TreeSelectionEvent tse) {
		this.currentNode = (TreeNode) tse.getPath().getLastPathComponent();

		return this.currentNode;
	}

	public void showNewTree(TreeNode rootNode) {
		this.rootNode = rootNode;

		// set the new root node as root of the JTree...
		this.setModel(new DefaultTreeModel(this.rootNode));

		// expand all elements of the JTree
		this.expandTree(this.rootNode);

		// set first element of JTree as selected element...
		this.selectFirstChild();
	}

	public void selectFirstChild() {
		this.setSelectionPath(new TreePath(((DefaultMutableTreeNode) this.rootNode.getFirstChild()).getPath()));
	}

	public int getWidth() {
		return this.treeWidth;
	}

	public void addTreePath(String pageName, TreePath path) {
		this.treePathMapping.put(pageName, path);
	}

	public TreePath getTreePath(String pageName) {
		return this.treePathMapping.get(pageName);
	}

	public TreeNode getCurrentNode() {
		return this.currentNode;
	}

	private void expandTree(DefaultMutableTreeNode node) {
		this.scrollPathToVisible(new TreePath(node.getPath()));

		for(int i = 0; i < node.getChildCount(); i += 1) {
			this.expandTree((DefaultMutableTreeNode) node.getChildAt(i));
		}
	}
}